Imports System.IO
''' <summary>
''' Basisklasse fr eine Datei
''' </summary>
Public MustInherit Class Datei
    Implements IDisposable

    ' Diese Klasse enthlt die Basis-Funktionen zur Verwaltung von Datei-Inhalten 
    ' Sie implementiert IDisposable zum Bereinigen von temporr angelegten Inhalten

    Public Event FormText_Changed()

#Region " Private Felder "

    ' Aktueller Pfad, unter dem die Datei gespeichert wird.
    ' Fr neue Datei zunchst leer.
    Private m_Pfad As String = ""

    ' Flag fr nur lesbare Datei (sie kann unter dem aktuellen Pfad nicht gespeichert werden)
    Private m_IsReadOnly As Boolean = False

    ' Datei-Name, der oben auf der Windows Form erscheint
    ' entweder realer Dateiname ohne Extension oder Dokument1, Dokument2... 
    Private m_DisplayName As String = ""

    ' Text, der oben auf einer Windows Form als Text erscheint 
    ' inclusive Information zum Schreibschutz
    Private m_FormText As String = ""

    ' Temporrer Pfad, unter dem die Datei beim Start und nach jedem Speichern
    ' zwischengespeichert wird. Diese temporre Datei dient zum Feststellen 
    ' von Vernderungen, um festzustellen, ob erneutes Speichern notwendig ist.
    ' Die temporre Datei wird bei Dispose gelscht.
    Private m_TempPath As String

    ' Flag, um mehrfaches Dispose zu vermeiden
    Private m_IsDisposed As Boolean = False

#End Region

#Region " Properties "

    ''' <summary>
    ''' Text, der oben auf einer Windows Form als Text erscheint 
    ''' inclusive Information zum Schreibschutz
    ''' </summary>
    ReadOnly Property FormText() As String
        Get
            Return m_FormText
        End Get
    End Property

    ''' <summary>
    ''' Aktueller Pfad, unter dem die Datei gespeichert wird.
    ''' </summary>
    ''' <remarks>Fr neue Datei zunchst leer.</remarks>
    Property Pfad() As String
        Get
            Return m_Pfad
        End Get
        Set(ByVal value As String)
            If m_Pfad <> value Then
                m_Pfad = value
                ' wenn die Datei einen eigenen Pfad erhlt, wird als Anzeigename
                ' der Dateiname verwendet
                m_DisplayName = IO.Path.GetFileNameWithoutExtension(m_Pfad)
                ' Text fr die Windows Form neu ermitteln
                RefreshTopText()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Flag fr nur lesbare Datei
    ''' </summary>
    Property IsReadOnly() As Boolean
        Get
            Return m_IsReadOnly
        End Get
        Set(ByVal value As Boolean)
            If m_IsReadOnly <> value Then
                m_IsReadOnly = value
                ' Text fr die Windows Form neu ermitteln
                RefreshTopText()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Datei-Name, der oben auf der Windows Form erscheint
    ''' </summary>
    ''' <remarks>ist entweder der reale Dateiname ohne Extension oder Dokument1, Dokument2... 
    ''' Zustzlich enthlt FormText noch eine Information zum Schreibschutz</remarks>
    Property DisplayName() As String
        Get
            Return m_DisplayName
        End Get
        Set(ByVal value As String)
            If m_DisplayName <> value Then
                m_DisplayName = value
                ' Text fr die Windows Form neu ermitteln
                RefreshTopText()
            End If
        End Set
    End Property

#End Region

#Region " Konstruktor "

    ''' <summary>
    ''' Konstruktor fr neue Datei
    ''' </summary>
    ''' <param name="FileName">Pfad zur Datei</param>
    ''' <remarks>Wenn fileName leer ist, wird eine neue Datei erstellt</remarks>
    Public Sub New(ByVal FileName As String)
        m_Pfad = FileName
        ' Wenn ein Pfad bergeben wird,
        If FileName <> "" Then
            ' wird als anzeige-Text fr die Windows Form der Dateiname ohne Extension verwendet
            m_DisplayName = IO.Path.GetFileNameWithoutExtension(FileName)
            ' Text fr die Windows Form neu ermitteln
            RefreshTopText()
        End If
        ' eindeutigen temporren Pfad fr die Datei festlegen
        m_TempPath = IO.Path.GetTempFileName
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' ermittelt den Text fr die Windows Form neu
    ''' </summary>
    Private Sub RefreshTopText()
        Dim TmpText As String = m_DisplayName
        If m_IsReadOnly Then TmpText &= " (schreibgeschtzt)"

        If m_FormText <> TmpText Then
            m_FormText = TmpText
            RaiseEvent FormText_Changed()
        End If

    End Sub

    ''' <summary>
    ''' Implementierung von IDisposable
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not m_IsDisposed Then
            If disposing Then
                ' hier verwaltete Ressourcen freigeben, wenn sie explizit aufgerufen werden
            End If
            ' Gemeinsam genutzte nicht verwaltete Ressourcen freigeben
            ' temporre Datei lschen
            If IO.File.Exists(m_TempPath) Then
                IO.File.Delete(m_TempPath)
            End If
        End If
        m_IsDisposed = True
    End Sub

    ''' <summary>
    ''' speichert die Datei neu temporr 
    ''' </summary>
    Public Sub Zustand_Speichern()
        Speichern(m_TempPath)
    End Sub

    Public Function Veraendert() As Boolean
        ' neuen eindeutigen temporren Pfad fr Datei-Vergleich besorgen
        Dim VergleichsDateiPfad As String = IO.Path.GetTempFileName
        ' Datei als Vergleichs-Datei unter diesem Pfad zwischenspeichern
        Speichern(VergleichsDateiPfad)

        ' File-Infos zum Vergleichen der Datei-Gre
        Dim FIbekannt As New FileInfo(m_TempPath)
        Dim FIVergleich As New FileInfo(VergleichsDateiPfad)

        If FIbekannt.Length <> FIVergleich.Length Then
            ' wenn beide Dateien unterschidlich gross sind, 
            ' temporre Vergleichs-Datei lschen und true zurckgeben
            IO.File.Delete(VergleichsDateiPfad)
            Return True
        Else
            ' wenn die Dateien gleich gro sein sollten, Hash-Werte vergleichen
            Dim MyHash As Security.Cryptography.HashAlgorithm = Security.Cryptography.HashAlgorithm.Create

            ' FileStream fr zuletzt zwischengespeicherte Datei
            Dim fs As New FileStream(m_TempPath, FileMode.Open, FileAccess.Read)
            ' Hash fr zuletzt zwischengespeicherte Datei
            Dim HashBekannt() As Byte = MyHash.ComputeHash(fs)
            fs.Close()

            ' FileStream fr Vergleichs-Datei
            fs = New FileStream(VergleichsDateiPfad, FileMode.Open, FileAccess.Read)
            ' Hash fr aktuell zwischengespeicherte Datei
            Dim HashVergleich() As Byte = MyHash.ComputeHash(fs)
            fs.Close()

            ' temporre Vergleichs-Datei lschen 
            IO.File.Delete(VergleichsDateiPfad)
            ' Ergebnis zurckgeben
            Return BitConverter.ToString(HashBekannt) <> BitConverter.ToString(HashVergleich)
        End If
    End Function

#End Region

#Region " MustOverride-Methoden "

    ''' <summary>
    ''' speichert die Datei unter angegebenem Pfad
    ''' </summary>
    MustOverride Function Speichern(ByVal Pfad As String) As Boolean

    ''' <summary>
    ''' liefert Information, ob die Datei leer ist
    ''' </summary>
    MustOverride Function IsEmpty() As Boolean

#End Region

End Class

