Imports System.Windows.Forms
''' <summary>
''' Spezieller Dialog zum Datei-ffnen unter Bercksichtigung, dass schon offene 
''' Dateien nur schreibgeschtzt ein zweites Mal geffnet werden drfen
''' </summary>
Public Class OpenDialog

#Region " Private Felder "

    ' Referenz zur DateiDefinition, um Messages mit geeigneten Begriffen zu versehen
    Private m_Def As DateiDefinition
    ' OpenFileDialog als klassenweit sichtbares Objekt (mit Events)
    Private WithEvents m_Dialog As OpenFileDialog
    ' Info, ob der Dialog mit Cancel geschlossen wurde
    Private m_DialogAbort As Boolean = False
    ' Info, ob die gleiche Datei, die im Dialog ausgewhlt wurde, schon einmal geffnet war
    Private m_FileAlreadyOpen As Boolean = False
    ' Liste mit allen offenen Dateien 
    Private m_OpenFiles As List(Of Datei)

#End Region

#Region " Properties "

    ''' <summary>
    ''' im Dialog ausgewhlter Dateiname
    ''' </summary>
    ReadOnly Property FileName() As String
        Get
            Return m_Dialog.FileName
        End Get
    End Property

    ''' <summary>
    ''' Info, ob die gleiche Datei, die im Dialog ausgewhlt wurde, schon einmal geffnet war
    ''' </summary>
    ReadOnly Property FileAlreadyOpen() As Boolean
        Get
            Return m_FileAlreadyOpen
        End Get
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New(ByVal Definition As DateiDefinition, ByVal OpenFiles As List(Of Datei))
        m_Def = Definition
        ' neuen OpenfileDialog mit Werten aus der Datei-Definition erstellen
        m_Dialog = New OpenFileDialog
        m_Dialog.Title = m_Def.AppTitle & " " & m_Def.DocumentName & " ffnen"
        m_Dialog.Filter = m_Def.AppTitle & " " & m_Def.DocumentsName & " (*" & m_Def.Extension & ")|*" & m_Def.Extension
        m_Dialog.FilterIndex = 1
        m_Dialog.RestoreDirectory = True
        m_OpenFiles = OpenFiles
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' zeigt den OpenFileDialog an und gibt True zurck, wenn nicht abgebrochen wurde 
    ''' </summary>
    Public Function ShowDialog_Erfolgreich() As Boolean
        Dim Result As DialogResult = m_Dialog.ShowDialog
        If m_DialogAbort Then Return False
        If Result = DialogResult.OK Then
            Return True
        End If
        Return False
    End Function

    ''' <summary>
    ''' Eventhandler fr das FileOk-Ereignis des OpenFileDialog
    ''' </summary>
    Private Sub m_Dialog_FileOk(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles m_Dialog.FileOk

        ' Wenn eine Datei mit gleichem Namen bereits geffnet ist
        If IsFileOpen(m_Dialog.FileName) Then
            m_FileAlreadyOpen = True
            ' Rckfrage, ob schreibgeschtzt geffnet werden soll
            Dim Result As DialogResult = MessageBox.Show(m_Def.Txt_DieseDatei & " ist bereits geffnet." & vbCrLf & "Mchten Sie " & m_Def.Txt_sie & " als schreibgeschtzte Kopie ffnen?", m_Def.AppTitle, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question)
            Select Case Result
                Case DialogResult.Yes
                    ' wenn ja, kann das Dialogfeld geschlossen werden
                    e.Cancel = False
                Case DialogResult.No
                    ' wenn nein, bleibt das Dialogfeld offen
                    e.Cancel = True
                Case DialogResult.Cancel
                    ' wenn Cancel, wird das Dialogfeld geschlossen, aber 
                    ' Flag m_DialogAbort wird auf True gesetzt
                    m_DialogAbort = True
                    e.Cancel = False
            End Select
        Else
            ' wenn noch keine gleichnamige Datei offen ist, wird das Dialogfeld regulr geschlossen
            m_FileAlreadyOpen = False
            e.Cancel = False
        End If

    End Sub

    ''' <summary>
    ''' ermittelt, ob eine Datei mit angegebenem Pfad geffnet ist
    ''' </summary>
    ''' <param name="FileName">Pfad zur Datei</param>
    Private Function IsFileOpen(ByVal FileName As String) As Boolean
        Dim LowerName As String = FileName.ToLower

        ' Feststellen, ob eine Windows Form mit gleichem Dateinamen wie im Dialogfeld ausgewhlt
        ' bereits geffnet ist
        For Each File As Datei In m_OpenFiles
            If File.Pfad.ToLower = LowerName AndAlso (Not File.IsReadOnly) Then
                Return True
            End If
        Next

        Return False
    End Function

#End Region

End Class
