Imports System.Windows.Forms
Public Class SaveDialog

#Region " Private Felder "

    ' Referenz zur DateiDefinition, um Messages mit geeigneten Begriffen zu versehen
    Private m_Def As DateiDefinition
    ' Referenz zur zu speichernden Datei
    Private m_File As Datei
    ' SaveFileDialog als klassenweit sichtbares Objekt (mit Events)
    Private WithEvents m_Dialog As SaveFileDialog
    ' Info, ob der Dialog mit Cancel geschlossen wurde
    Private m_DialogAbort As Boolean = False
    ' Liste mit allen offenen Dateien 
    Private m_OpenFiles As List(Of Datei)

#End Region

#Region " Properties "

    ''' <summary>
    ''' im Dialog ausgewhlter Dateiname
    ''' </summary>
    ReadOnly Property FileName() As String
        Get
            Return m_Dialog.FileName
        End Get
    End Property


#End Region

#Region " Konstruktor "

    ''' <summary>
    ''' Konstruktor mit Referenz zur zu speichernden Datei und DateiDefinition
    ''' </summary>
    ''' <param name="File">Referenz zur zu speichernden Datei</param>
    ''' <param name="Def">DateiDefinition</param>
    Public Sub New(ByVal File As Datei, ByVal Def As DateiDefinition, ByVal OpenFiles As List(Of Datei))
        m_File = File
        m_Def = Def

        ' neuen SaveFileDialog mit Werten aus der Datei-Definition erstellen
        m_Dialog = New SaveFileDialog
        m_Dialog.Title = m_Def.AppTitle & " " & m_Def.DocumentName & " speichern"
        m_Dialog.Filter = m_Def.AppTitle & " " & m_Def.DocumentsName & " (*" & m_Def.Extension & ")|*" & m_Def.Extension
        m_Dialog.FilterIndex = 1
        m_Dialog.RestoreDirectory = True
        m_Dialog.AddExtension = True
        ' OverwritePrompt wird False, weil die Fehlermeldungen hier selbst generiert werden
        m_Dialog.OverwritePrompt = False

        If File.Pfad <> "" Then
            'Diese Datei existiert bereits
            m_Dialog.FileName = File.Pfad
        Else
            'Diese Datei wurde noch nie gespeichert

            ' als Standard-Ordner wird der in der Dateidefinition angegebene PersonalFolderName-Ordner verwendet
            ' Wenn dieser Ordner im Documents-Directory nicht existiert, wird er erstellt
            Dim PersonalFolderPath As String = Environment.GetFolderPath(Environment.SpecialFolder.Personal) & "\" & m_Def.PersonalFolderName
            If Not IO.Directory.Exists(PersonalFolderPath) Then
                IO.Directory.CreateDirectory(PersonalFolderPath)
            End If
            m_Dialog.FileName = PersonalFolderPath & "\" & File.DisplayName
        End If
        m_OpenFiles = OpenFiles
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' zeigt den SaveFileDialog an und gibt True zurck, wenn nicht abgebrochen wurde 
    ''' </summary>
    Public Function ShowDialog_Erfolgreich() As Boolean
        Dim Result As DialogResult = m_Dialog.ShowDialog
        If m_DialogAbort Then Return False
        If Result = DialogResult.OK Then
            Return True
        End If
        Return False
    End Function


    ''' <summary>
    ''' Eventhandler fr das FileOk-Ereignis des SaveFileDialog
    ''' </summary>
    Private Sub m_Dialog_FileOk(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles m_Dialog.FileOk

        If m_File.Pfad <> m_Dialog.FileName Then
            ' wenn im Dialog ein anderer Dateiname als der bisherige ausgewhlt wurde
            If IsFileOpen(m_Dialog.FileName) Then
                ' wenn im Dialog ein existierender Dateiname ausgewhlt wurde, und diese Datei bereits
                ' geffnet ist: Meldung anzeigen und Dialog offen lassen
                MessageBox.Show("Es ist bereits " & m_Def.Txt_eineDatei & " mit diesem Namen geffnet." & vbCrLf & _
                "Bitte schlieen Sie " & m_Def.Txt_diegeoeffneteDatei & " oder whlen Sie einen anderen Dateinamen aus.", m_Def.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                e.Cancel = True
            ElseIf IO.File.Exists(m_Dialog.FileName) Then
                ' wenn im Dialog ein existierender Dateiname ausgewhlt wurde, und diese Datei bereits
                ' geffnet ist: Frage, ob berschrieben werden soll
                Dim Result As DialogResult = MessageBox.Show("Es existiert bereits " & m_Def.Txt_eineDatei & " mit diesem Namen." & vbCrLf & _
                           "Mchten Sie " & m_Def.Txt_sie & " berschreiben?", m_Def.AppTitle, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Information)
                Select Case Result
                    Case DialogResult.Yes
                        ' wenn ja, kann der Dialog geschlossen werden
                        e.Cancel = False
                    Case DialogResult.No
                        ' wenn nein, bleibt der Dialog offen
                        e.Cancel = True
                    Case DialogResult.Cancel
                        ' wenn Abbrechen, wird der Dialog geschlossen und 
                        ' m_DialogAbort auf True gesetzt
                        m_DialogAbort = True
                        e.Cancel = False
                End Select
            End If
        ElseIf m_File.IsReadOnly Then
            ' wenn der ausgewhlte Dateiname gleich zum bisherigen ist und die Datei 
            ' schreibgeschtzt geffnet war, darf sie unter diesem Namen nicht 
            ' gespeichert werden. Also wird darauf hingewiesen und das Dialogfeld bleibt offen.
            MessageBox.Show(m_Def.Txt_DieseDatei & " wurde schreibgeschtzt geffnet." & vbCrLf & _
            "Bitte speichern Sie " & m_Def.Txt_sie & " unter einem anderen Namen.", m_Def.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            e.Cancel = True
        End If

    End Sub

    ''' <summary>
    ''' ermittelt, ob eine Datei mit angegebenem Pfad geffnet ist
    ''' </summary>
    ''' <param name="FileName">Pfad zur Datei</param>
    Private Function IsFileOpen(ByVal FileName As String) As Boolean
        Dim LowerName As String = FileName.ToLower

        ' Feststellen, ob eine Windows Form mit gleichem Dateinamen wie im Dialogfeld ausgewhlt
        ' bereits geffnet ist
        For Each File As Datei In m_OpenFiles
            If File.Pfad.ToLower = LowerName AndAlso (Not File.IsReadOnly) Then
                Return True
            End If
        Next

        Return False
    End Function

#End Region

End Class
