Imports System.Xml
Imports Dateiverwaltung
''' <summary>
''' Eine einfache Dummy-Datei zur Demonstration
''' </summary>
''' <remarks>In diesem Fall enthlt die Datei 50 mal 50 Pixel,
''' fr jeden Pixel sind 8 Farbindizes mglich (es htte fr jeden Pixel auch
''' nicht unbedingt ein Integer-Wert sein mssen.)</remarks>
Public Class BildDatei
    Inherits Datei
    ' erbt von der Klasse Datei, die erledigt eigentlich die Hauptarbeit.

#Region " Private Felder "
    ' 50 x 50 Pixel
    Private m_ColorIndizes(49, 49) As Integer

#End Region

#Region " Properties "

    ''' <summary>
    ''' liefert das enthaltene Array mit Farb-Indizes
    ''' </summary>
    ReadOnly Property ColorIndizes() As Integer(,)
        Get
            Return m_ColorIndizes
        End Get
    End Property

#End Region

#Region " Konstruktor "

    ''' <summary>
    ''' Konstrktor
    ''' </summary>
    ''' <param name="FileName">Dateiname</param>
    ''' <remarks>Der Dateiname kann auch leer sein, dann wird eine neue Datei erstellt</remarks>
    Public Sub New(ByVal FileName As String)
        MyBase.New(FileName)
        ' Wenn ein Dateiname bergeben wird, muss auch eine Datei existieren
        If FileName <> "" Then
            ' dann wird die Datei gelesen
            ReadXML(FileName)
        End If
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' liest die Farbwerte aus einer XML-Datei
    ''' </summary>
    ''' <param name="Path">Dateiname</param>
    Private Sub ReadXML(ByVal Path As String)
        ' XmlTextReader definieren
        Dim reader As New XmlTextReader(Path)
        reader.WhitespaceHandling = WhitespaceHandling.None

        ' XML-Dokument erstellen und ber den Reader laden
        Dim xmlDoc As XmlDocument = New XmlDocument
        xmlDoc.Load(reader)
        ' Reader kann geschlossen werden
        reader.Close()

        Dim xnod As XmlNode = xmlDoc.DocumentElement
        Dim I, J As Integer

        ' jeder XML-Node, der den Namen "Col" hat, enthlt
        ' auch einen Farbwert und Index fr Zeile und Spalte
        ' (ginge zwar deutlich sparsamer, habe ich aber zum besseren Verstndnis 
        ' so gemacht)
        For Each Nd As XmlNode In xnod.ChildNodes
            If Nd.Name = "Col" Then
                I = CInt(Nd.Attributes("x").Value)
                J = CInt(Nd.Attributes("y").Value)
                m_ColorIndizes(I, J) = CInt(Nd.Attributes("idx").Value)
            End If
        Next
    End Sub

    ''' <summary>
    ''' speichert Farbwerte in einer XML-Datei
    ''' </summary>
    ''' <param name="Path">Pfad, unter dem zu speichern ist</param>
    Public Overrides Function Speichern(ByVal Path As String) As Boolean
        'XML Schreiben

        ' XmlTextWriter devfinieren
        Dim writer As XmlTextWriter = New XmlTextWriter(Path, System.Text.Encoding.UTF8)
        writer.WriteStartDocument(True)

        writer.WriteStartElement("Elemente")

        ' im weiteren alle Elemente so schreiben, dass sie 
        ' spter mit dem XML-Reader wieder gelesen werden knnen
        For I As Integer = 0 To 49
            For J As Integer = 0 To 49
                writer.WriteStartElement("Col")
                writer.WriteAttributeString("x", I.ToString)
                writer.WriteAttributeString("y", J.ToString)
                writer.WriteAttributeString("idx", m_ColorIndizes(I, J).ToString)
                writer.WriteEndElement()
            Next
        Next

        writer.WriteEndElement()
        writer.WriteEndDocument()
        writer.Close()
        Return True
    End Function

    ''' <summary>
    ''' Funktion zum Testen, ob die Bilddatei ohne Inhalt ist
    ''' (der Test zum berprfen, ob sie gendert ist, ist in der Basisklasse enthalten)
    ''' </summary>
    Public Overrides Function IsEmpty() As Boolean
        ' wenn alle Farbindizes 0 sind (entspricht weiss)
        ' wird True zurckgegeben
        For I As Integer = 0 To 49
            For J As Integer = 0 To 49
                If m_ColorIndizes(I, J) <> 0 Then
                    Return False
                End If
            Next
        Next
        Return True
    End Function

#End Region

End Class

