Imports System.ComponentModel
Imports System.Drawing
''' <summary>
''' ContainerControl, in dem Inhalt ausgerichtet und zentriert werden kann,
''' das bei Notwendigkeit ScrollBars anzeigt und sich beim Resize 
''' daran orientiert, die zuletzt mit Scrollen erreichte Inhalts-Position 
''' beizubehalten
''' </summary>
Public Class AlignContainer
    Inherits ContainerControl

#Region " Private Variablen "

    ' dies sind ein paar Rahmenlinien
    ' ging leider nur mit Controls. Damit erscheint der Rahmen dieses
    ' Controls ber den Children. Ein Handler fr das Paint-Event der 
    ' Children und dabei auf den Children zeichnen fhrt nicht zu einem
    ' befriedigenden Ergebnis
    Private m_LineControl1 As LineControl
    Private m_LineControl2 As LineControl
    Private m_LineControl3 As LineControl
    Private m_LineControl4 As LineControl

    ' Panel fr horizontaler ScrollBar und ScrollBarDivider
    Private m_BottomScrollBarPanel As Panel
    ' das kleine Rechteck, das zwei sichtbare ScrollBars trennt
    Private m_ScrollBarDivider As Panel

    ' horizontale ScrollBar
    Private m_HBar As HScrollBar
    ' vertikale ScrollBar
    Private m_VBar As VScrollBar



    ' Diese Klassen regeln die vertikale und horizontale Ausrichtung des Inhalts 
    ' und das Anpassen der ScrollBar-Werte. Sie sorgen fr das Anzeigen der ScrollBars, 
    ' liefern Properties, die fr abgeleitete Controls ntzlich sind. Dazu hier im 
    ' AlignContainer die Properties ContentBackRectangle, ContentRectangle und ValuesRectangle ansehen.
    Private WithEvents m_VCalculator As ScrollRangeCalculator
    Private WithEvents m_HCalculator As ScrollRangeCalculator

    ' Liste mit allen zum AlignContainer gehrenden Controls  
    Private m_OwnControls As List(Of Control)

    ' Dictionary mit manuell hinzu gefgten Controls und ihren Positionen
    Private m_ContentControls As Dictionary(Of Control, Point)

    ' aktuelle vertikale und horizontale Ausrichtung 
    Private m_ContentAlign As ContentAlignment = ContentAlignment.TopLeft

    ' Flag zum Vermeiden von unerwnschten rekursiven Aufrufen fr die Methode CalculatePositions
    Private m_Recalculating As Boolean = False

#End Region

#Region " Properties "

    <Category("AlignContainer"), Description("Gibt die Ausrichtung enthaltener Controls zurck " & _
    "oder legt diese fest.")> _
    Property ContentAlignment() As ContentAlignment
        Get
            Return m_ContentAlign
        End Get
        Set(ByVal value As ContentAlignment)
            If m_ContentAlign <> value Then
                m_ContentAlign = value
                ' Align fr HCalculator einstellen - kann linksbndig, zentriert oder rechtsbndig sein
                Select Case m_ContentAlign
                    Case ContentAlignment.TopLeft, ContentAlignment.MiddleLeft, ContentAlignment.BottomLeft
                        m_HCalculator.Alignment = StringAlignment.Near
                    Case ContentAlignment.TopCenter, ContentAlignment.MiddleCenter, ContentAlignment.BottomCenter
                        m_HCalculator.Alignment = StringAlignment.Center
                    Case Else
                        m_HCalculator.Alignment = StringAlignment.Far
                End Select
                ' Align fr VCalculator einstellen - kann obenbndig, zentriert oder untenbndig sein
                Select Case m_ContentAlign
                    Case ContentAlignment.TopLeft, ContentAlignment.TopCenter, ContentAlignment.TopRight
                        m_VCalculator.Alignment = StringAlignment.Near
                    Case ContentAlignment.MiddleLeft, ContentAlignment.MiddleCenter, ContentAlignment.MiddleRight
                        m_VCalculator.Alignment = StringAlignment.Center
                    Case Else
                        m_VCalculator.Alignment = StringAlignment.Far
                End Select
                ' Child-Controls neu positionieren
                SetOnlyPositions()
            End If
        End Set
    End Property

    <Category("AlignContainer"), Description("Zentriert den Inhalt horizontal, wenn das Control breiter als der Inhalt ist.")> _
    Property CenterSmallHorizontal() As Boolean
        Get
            Return m_HCalculator.CenterSmallContent
        End Get
        Set(ByVal value As Boolean)
            If m_HCalculator.CenterSmallContent <> value Then
                m_HCalculator.CenterSmallContent = value
                ' Child-Controls neu positionieren
                SetOnlyPositions()
            End If
        End Set
    End Property

    <Category("AlignContainer"), Description("Zentriert den Inhalt vertikal, wenn das Control hher als der Inhalt ist.")> _
    Property CenterSmallVertikal() As Boolean
        Get
            Return m_VCalculator.CenterSmallContent
        End Get
        Set(ByVal value As Boolean)
            If m_VCalculator.CenterSmallContent <> value Then
                m_VCalculator.CenterSmallContent = value
                ' Child-Controls neu positionieren
                SetOnlyPositions()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Gibt ein Rechteck zurck, das Position und Gre des aktuell im Control sichtbaren 
    ''' Inhalts-Ausschnittes beschreibt
    ''' </summary>
    <Browsable(False)> _
    ReadOnly Property ContentBackRectangle() As Rectangle
        Get
            Dim HBackRange As Range = m_HCalculator.ContentBackGroundRange
            Dim VBackRange As Range = m_VCalculator.ContentBackGroundRange
            Return New Rectangle(HBackRange.Start, VBackRange.Start, HBackRange.Length, VBackRange.Length)
        End Get
    End Property

    ''' <summary>
    ''' Gibt ein Rechteck zurck, das Position und Gre des gesamten im Control enthaltenen
    ''' Inhalts beschreibt
    ''' </summary>
    <Browsable(False)> _
    ReadOnly Property ContentRectangle() As Rectangle
        Get
            Dim HRange As Range = m_HCalculator.ContentRange
            Dim VRange As Range = m_VCalculator.ContentRange
            Return New Rectangle(HRange.Start, VRange.Start, HRange.Length, VRange.Length)
        End Get
    End Property

    ''' <summary>
    ''' Gibt ein Rechteck zurck, das den Wertbereich (Start-Pixel und Lnge in Pixeln) 
    ''' des aktuell im Control sichtbaren Inhalts-Ausschnittes beschreibt
    ''' </summary>
    <Browsable(False)> _
    ReadOnly Property ValuesRectangle() As Rectangle
        Get
            Dim HVisibleRange As Range = m_HCalculator.VisibleValuesRange
            Dim VVisibleRange As Range = m_VCalculator.VisibleValuesRange
            Return New Rectangle(HVisibleRange.Start, VVisibleRange.Start, HVisibleRange.Length, VVisibleRange.Length)
        End Get
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New()
        m_BottomScrollBarPanel = New Panel
        m_ScrollBarDivider = New Panel
        m_HBar = New HScrollBar
        m_VBar = New VScrollBar
        m_LineControl4 = New LineControl
        m_LineControl3 = New LineControl
        m_LineControl2 = New LineControl
        m_LineControl1 = New LineControl

        ' Damit sind die eigenen Controls definiert. Diese werden einer 
        ' Liste hinzugefgt, um spter eigene Controls von fremden zu unterscheiden.
        m_OwnControls = New List(Of Control)
        m_OwnControls.Add(m_BottomScrollBarPanel)
        m_OwnControls.Add(m_ScrollBarDivider)
        m_OwnControls.Add(m_HBar)
        m_OwnControls.Add(m_VBar)
        m_OwnControls.Add(m_LineControl4)
        m_OwnControls.Add(m_LineControl3)
        m_OwnControls.Add(m_LineControl2)
        m_OwnControls.Add(m_LineControl1)

        ' Dictionary mit Fremd-Controls und deren Positionen
        m_ContentControls = New Dictionary(Of Control, Point)

        ' Layout-Logik unterbrechen
        m_BottomScrollBarPanel.SuspendLayout()
        SuspendLayout()

        ' Das her sieht wie im Windows Forms Designer aus. 
        ' Stammt auch von dort. Um das Control ordentlich zusammenzusetzen,
        ' habe ich zuerst ein UserControl erstellt. Der Vorteil ist,
        ' dass hier vor erster Anzeige des Controls Gren und Positionen
        ' exakt definiert sind.
        m_BottomScrollBarPanel.Controls.Add(m_HBar)
        m_BottomScrollBarPanel.Controls.Add(m_ScrollBarDivider)
        m_BottomScrollBarPanel.Dock = DockStyle.Bottom
        m_BottomScrollBarPanel.Location = New System.Drawing.Point(1, 82)
        m_BottomScrollBarPanel.Size = New System.Drawing.Size(149, 17)
        m_BottomScrollBarPanel.Visible = False

        'ScrollBarDivider
        m_ScrollBarDivider.Dock = DockStyle.Right
        m_ScrollBarDivider.Location = New System.Drawing.Point(132, 0)
        m_ScrollBarDivider.Size = New System.Drawing.Size(17, 17)

        'HBar
        m_HBar.Dock = DockStyle.Fill
        m_HBar.Location = New System.Drawing.Point(0, 0)
        m_HBar.Size = New System.Drawing.Size(132, 17)

        'VBar
        m_VBar.Dock = DockStyle.Right
        m_VBar.Location = New System.Drawing.Point(133, 1)
        m_VBar.Size = New System.Drawing.Size(17, 81)
        m_VBar.Visible = False

        'LineControl4
        m_LineControl4.Align = BorderAlignments.Right
        m_LineControl4.Dock = DockStyle.Right
        m_LineControl4.Location = New System.Drawing.Point(150, 1)
        m_LineControl4.Size = New System.Drawing.Size(1, 98)

        'LineControl3
        m_LineControl3.Align = BorderAlignments.Left
        m_LineControl3.Dock = DockStyle.Left
        m_LineControl3.Location = New System.Drawing.Point(0, 1)
        m_LineControl3.Size = New System.Drawing.Size(1, 98)

        'LineControl2
        m_LineControl2.Align = BorderAlignments.Top
        m_LineControl2.Dock = DockStyle.Top
        m_LineControl2.Location = New System.Drawing.Point(0, 0)
        m_LineControl2.Size = New System.Drawing.Size(151, 1)

        'LineControl1
        m_LineControl1.Align = BorderAlignments.Bottom
        m_LineControl1.Dock = DockStyle.Bottom
        m_LineControl1.Location = New System.Drawing.Point(0, 99)
        m_LineControl1.Size = New System.Drawing.Size(151, 1)


        AutoScaleDimensions = New System.Drawing.SizeF(6.0!, 13.0!)
        AutoScaleMode = AutoScaleMode.Font
        Controls.Add(m_VBar)
        Controls.Add(m_BottomScrollBarPanel)
        Controls.Add(m_LineControl4)
        Controls.Add(m_LineControl3)
        Controls.Add(m_LineControl2)
        Controls.Add(m_LineControl1)

        ' HCalculator und VCalculator, die die ScrollBar- und Inhaltspositionen ermitteln,
        ' initialisieren
        m_VCalculator = New ScrollRangeCalculator(100, 1, m_VBar, m_VBar, m_BottomScrollBarPanel, m_BottomScrollBarPanel.Height, m_ScrollBarDivider)
        m_HCalculator = New ScrollRangeCalculator(151, 1, m_HBar, m_BottomScrollBarPanel, m_VBar, m_VBar.Width, m_ScrollBarDivider)

        Size = New System.Drawing.Size(151, 100)
        ' Layout-Logik wieder aufnehmen
        m_BottomScrollBarPanel.ResumeLayout(False)
        ResumeLayout(False)

    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Methode bei Hinzufgen von Controls zum AlignContainer
    ''' </summary>
    Protected Overrides Sub OnControlAdded(ByVal e As System.Windows.Forms.ControlEventArgs)
        MyBase.OnControlAdded(e)

        If Not m_OwnControls.Contains(e.Control) Then
            ' Hier wird dafr gesorgt, dass auf Events, die die enthaltenen Controls liefern,
            ' reagiert werden kann

            ' Control in einer eigenen Liste unterbringen
            m_ContentControls.Add(e.Control, New Point(0, 0))


            ' Handler fr nderung der Position des Controls - der wird verwendet, damit wir die 
            ' Position auf unsere gewnschten Werte zurck setzen knnen
            AddHandler e.Control.LocationChanged, AddressOf Child_LocationChanged
            ' Handler fr Grennderung - auch hier wollen wir mit neuer Positionierung der Controls reagieren
            AddHandler e.Control.SizeChanged, AddressOf Child_SizeChanged

            ' Maximale Gre aller enthaltenen Child-Controls ermitteln
            ' und Control-Positionen neu verteilen
            CalculateBoundsAndSetPositions()
        End If
    End Sub

    ''' <summary>
    ''' Methode bei Entfernen eines Controls aus dem AlignContainer
    ''' </summary>
    Protected Overrides Sub OnControlRemoved(ByVal e As System.Windows.Forms.ControlEventArgs)
        ' alle ursprnglich hinzugefgten Eventhandler werden wieder entfernt
        RemoveHandler e.Control.LocationChanged, AddressOf Child_LocationChanged
        RemoveHandler e.Control.SizeChanged, AddressOf Child_SizeChanged

        ' Control aus der eigenen Liste entfernen
        m_ContentControls.Remove(e.Control)
        ' Maximale Gre aller noch enthaltenen Child-Controls ermitteln
        ' und Control-Positionen neu verteilen
        CalculateBoundsAndSetPositions()
        ' Mehtode der Basisklasse aufrufen
        MyBase.OnControlRemoved(e)
    End Sub

    ''' <summary>
    ''' Methode bei ndern der Position eines Child-Controls
    ''' </summary>
    Private Sub Child_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs)
        ' passiert z.B. beim Versuch, ein Child-Control manuell an eine neue Position zu verschieben

        ' Wenn sich die Position whrend der Neuberechnung der Child-Control-Positionen
        ' ndert, nichts tun
        If m_Recalculating Then Exit Sub
        ' Ansonsten die festgelegte Control-Position wiederherstellen
        Dim Ctl As Control = sender
        Ctl.Location = m_ContentControls(Ctl)
    End Sub

    ''' <summary>
    ''' Methode bei ndern der Gre eines Child-Controls
    ''' </summary>
    Private Sub Child_SizeChanged(ByVal sender As Object, ByVal e As System.EventArgs)
        ' passiert z.B. bei manueller Grennderung eines Child-Controls im Windows Forms Designer

        ' Maximale Gre aller enthaltenen Child-Controls ermitteln
        ' und Control-Positionen neu verteilen
        CalculateBoundsAndSetPositions()
    End Sub

    ''' <summary>
    ''' Methode bei ndern der eigenen Control-Gre
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)

        ' nur VCalculator und HCalculator Breite bzw. Hhe des Controls mitteilen,
        ' sie ermitteln ScrollBar-Verschiebung und Position des Inhaltsrechtecks neu
        m_VCalculator.ControlWidth = Me.Height
        m_HCalculator.ControlWidth = Me.Width

        ' Child-Controls neu positionieren
        SetOnlyPositions()
    End Sub

    ''' <summary>
    ''' ermittelt die maximale Gre aller enthaltenen Child-Controls neu
    ''' und legt die Positionen der Child-Controls neu fest
    ''' </summary>
    Private Sub CalculateBoundsAndSetPositions()
        ' Layout-Logik unterbrechen
        Me.SuspendLayout()

        ' Schleifen durch Recalculating-Flag verhindern
        If m_Recalculating Then Exit Sub
        m_Recalculating = True

        ' maximale Gre aller enthaltenen Child-Controls neu ermitteln, diese 
        ' dem VCalculator bzw. HCalculator mitteilen
        CalculateBounds()
        ' Child-Controls neu positionieren
        SetControlPositions()

        m_Recalculating = False

        ' Layout-Logik fortsetzen
        Me.ResumeLayout(False)
    End Sub

    ''' <summary>
    ''' positioniert Child-Controls neu
    ''' </summary>
    Private Sub SetOnlyPositions()
        ' Layout-Logik unterbrechen
        Me.SuspendLayout()

        ' Schleifen durch Recalculating-Flag verhindern
        If m_Recalculating Then Exit Sub
        m_Recalculating = True

        ' Child-Controls neu positionieren
        SetControlPositions()

        m_Recalculating = False
        ' Layout-Logik fortsetzen
        Me.ResumeLayout(False)
    End Sub

    ''' <summary>
    ''' ermittelt die maximale Gre aller enthaltenen Child-Controls neu
    ''' und gibt diese an den VCalculator bzw. HCalculator weiter
    ''' </summary>
    Private Sub CalculateBounds()

        Dim NewWidth As Integer = 0
        Dim NewHeight As Integer = 0
        For Each Ctl As Control In m_ContentControls.Keys
            NewWidth = Math.Max(NewWidth, Ctl.Width)
            NewHeight = Math.Max(NewHeight, Ctl.Height)
        Next

        m_HCalculator.ContentLength = NewWidth
        m_VCalculator.ContentLength = NewHeight

    End Sub

    ''' <summary>
    ''' positioniert Child-Controls neu
    ''' </summary>
    Private Sub SetControlPositions()
        ' aktuelles Inhaltsrechteck von VCalculator und HCalculator holen
        Dim Rect As Rectangle = Me.ContentRectangle

        ' Hier Dictionarys zu verwenden ist vielleicht ein bisschen umstndlich
        Dim XPositions As New Dictionary(Of Control, Integer)
        Dim YPositions As New Dictionary(Of Control, Integer)


        Dim AllControls As New List(Of Control)
        For Each Ctl As Control In m_ContentControls.Keys
            AllControls.Add(Ctl)
        Next

        ' X-Positionen ermitteln
        If m_HCalculator.Alignment = StringAlignment.Near Then
            For Each Ctl As Control In AllControls
                XPositions.Add(Ctl, Rect.Left)
            Next
        ElseIf m_HCalculator.Alignment = StringAlignment.Center Then
            Dim Center As Single = m_HCalculator.ContentRange.Center
            For Each Ctl As Control In AllControls
                XPositions.Add(Ctl, CInt(Center - CSng(Ctl.Width) / 2.0F))
            Next
        ElseIf m_HCalculator.Alignment = StringAlignment.Far Then
            For Each Ctl As Control In AllControls
                XPositions.Add(Ctl, Rect.Right - Ctl.Width)
            Next
        End If

        ' Y-Positionen ermitteln
        If m_VCalculator.Alignment = StringAlignment.Near Then
            For Each Ctl As Control In AllControls
                YPositions.Add(Ctl, Rect.Top)
            Next
        ElseIf m_VCalculator.Alignment = StringAlignment.Center Then
            Dim Center As Single = m_VCalculator.ContentRange.Center
            For Each Ctl As Control In AllControls
                YPositions.Add(Ctl, CInt(Center - CSng(Ctl.Height) / 2.0F))
            Next
        ElseIf m_VCalculator.Alignment = StringAlignment.Far Then
            For Each Ctl As Control In AllControls
                YPositions.Add(Ctl, Rect.Bottom - Ctl.Height)
            Next
        End If

        ' die neuen Positionen sowohl im Dictionary vermerken als auch den Controls selber weiter geben 
        For Each Ctl As Control In AllControls
            Dim Pt As New Point(XPositions(Ctl), YPositions(Ctl))
            m_ContentControls(Ctl) = Pt
            Ctl.Location = Pt
        Next

    End Sub

    ''' <summary>
    ''' Eventhandler fr das Scrolled-Ereignis des HCalculators
    ''' </summary>
    Private Sub m_HCalculator_Scrolled() Handles m_HCalculator.Scrolled
        ' tritt ein nach manuellem Scrollen und folgender Berechnung im HCalculator
        ' Child-Controls neu positionieren
        SetOnlyPositions()
    End Sub

    ''' <summary>
    ''' Eventhandler fr das Scrolled-Ereignis des VCalculators
    ''' </summary>
    Private Sub m_VCalculator_Scrolled() Handles m_VCalculator.Scrolled
        ' tritt ein nach manuellem Scrollen und folgender Berechnung im VCalculator
        ' Child-Controls neu positionieren
        SetOnlyPositions()
    End Sub

#End Region

End Class
