''' <summary>
''' Control zum Zeichnen und berdecken von Rahmenlinien
''' </summary>
Public Class LineControl
    Inherits Control

#Region " Private Variablen "

    ' Ausrichtung
    Private m_Align As BorderAlignments
    ' Zeichen-Rechteck
    Private m_Rect As Rectangle

#End Region

#Region " Properties "

    ''' <summary>
    ''' Ausrichtung der Rahmenlinie
    ''' </summary>
    Property Align() As BorderAlignments
        Get
            Return m_Align
        End Get
        Set(ByVal value As BorderAlignments)
            m_Align = value
            ' Abhngig von Ausrichtung DockStyle neu festlegen
            Select Case m_Align
                Case BorderAlignments.Bottom
                    Me.Dock = DockStyle.Bottom
                Case BorderAlignments.Top
                    Me.Dock = DockStyle.Top
                Case BorderAlignments.Left
                    Me.Dock = DockStyle.Left
                Case BorderAlignments.Right
                    Me.Dock = DockStyle.Right
            End Select

            ' Abhngig von Ausrichtung Breite bzw. Hhe des Controls neu festlegen
            Select Case m_Align
                Case BorderAlignments.Bottom, BorderAlignments.Top
                    Me.Height = 1
                Case BorderAlignments.Left, BorderAlignments.Right
                    Me.Width = 1
            End Select
            ' Zeichenrechteck neu ermitteln. 
            GetRectangle()
        End Set
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New()
        Me.Align = BorderAlignments.Left
        ' DoubleBuffering aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
        Me.UpdateStyles()
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' ermittelt das Zeichenrechteck neu
    ''' </summary>
    Private Sub GetRectangle()
        Select Case m_Align
            Case BorderAlignments.Bottom
                m_Rect = New Rectangle(0, -2, Me.Width - 1, 2)
            Case BorderAlignments.Top
                m_Rect = New Rectangle(0, 0, Me.Width - 1, 2)
            Case BorderAlignments.Left
                m_Rect = New Rectangle(0, 0, 2, Me.Height - 1)
            Case BorderAlignments.Right
                m_Rect = New Rectangle(-2, 0, 2, Me.Height - 1)
        End Select
        ' Control neu zeichnen
        Invalidate()
    End Sub

    ' berschreiben der OnResize-Methode der Basisklasse
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Zeichenrechteck neu ermitteln und Control neu zeichnen
        GetRectangle()
    End Sub

    ' berschreiben der OnPaint-Methode der Basisklasse
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        ' hier wird zwar ein Rechteck gezeichnet, auf dem Control erscheint aber nur eine Linie.
        ControlPaint.DrawVisualStyleBorder(e.Graphics, m_Rect)
    End Sub

#End Region

End Class
