''' <summary>
''' beschreibt einen Wertebereich mit Start und Lnge
''' </summary>
Public Class Range

#Region " Private Variablen "

    ' Start
    Private m_Start As Integer = 0
    ' Lnge
    Private m_Length As Integer = 0
    ' Event fr nderung der Lnge
    Public Event LengthChanged()
    ' Event fr nderung der Startposition
    Public Event PositionChanged()

#End Region

#Region " Properties "

    ''' <summary>
    ''' Startposition
    ''' </summary>
    Property Start() As Integer
        Get
            Return m_Start
        End Get
        Set(ByVal value As Integer)
            If m_Start <> value Then
                m_Start = value
                RaiseEvent PositionChanged()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Lnge
    ''' </summary>
    Property Length() As Integer
        Get
            Return m_Length
        End Get
        Set(ByVal value As Integer)
            If m_Length <> value Then
                m_Length = value
                RaiseEvent LengthChanged()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Endposition
    ''' </summary>
    ReadOnly Property Ende() As Integer
        Get
            Return m_Start + m_Length
        End Get
    End Property

    ''' <summary>
    ''' Mittelpunkt zwischen Start- und Endposition
    ''' </summary>
    ReadOnly Property Center() As Single
        Get
            Return CSng(m_Start) + CSng(m_Length) / 2.0F
        End Get
    End Property

#End Region

#Region " Konstruktoren "
    Public Sub New()

    End Sub
    Public Sub New(ByVal Start As Integer)
        m_Start = Start
    End Sub
    Public Sub New(ByVal Start As Integer, ByVal Length As Integer)
        m_Start = Start
        m_Length = Length
    End Sub
#End Region

#Region " Methoden "

    ''' <summary>
    ''' gibt die Schnittmenge des aktuellen mit einem anderen Range-Objekt zurck
    ''' </summary>
    ''' <param name="Other">anderes Range-Objekt</param>
    ''' <returns>Range-Objekt</returns>
    Public Function Intersection(ByVal Other As Range) As Range
        Dim MinEnde As Integer = Math.Min(Other.Ende, Me.Ende)
        Dim MaxStart As Integer = Math.Max(Other.Start, Me.Start)
        If MinEnde > MaxStart Then
            Return New Range(MaxStart, MinEnde - MaxStart)
        Else
            Return New Range(0, 0)
        End If
    End Function

    ''' <summary>
    ''' Ruft ein Range-Objekt ab, das die Gesamtmenge zweier Range-Objekte enthlt 
    ''' </summary>
    ''' <param name="a">Range-Objekt</param>
    ''' <param name="b">Range-Objekt</param>
    ''' <returns>Range-Objekt</returns>
    Public Shared Function Union(ByVal a As Range, ByVal b As Range) As Range
        Dim MaxEnde As Integer = Math.Max(a.Ende, b.Ende)
        Dim MinStart As Integer = Math.Min(a.Start, b.Start)
        Return New Range(MinStart, MaxEnde - MinStart)
    End Function

    Public Overrides Function ToString() As String
        Return "Start: " & m_Start.ToString & " Length: " & m_Length.ToString
    End Function

#End Region

End Class