Public Class ScrollRangeCalculator
    ' die Kommentare hier entsprechen einer horizontalen ScrollBar 

#Region " Private Variablen "

    ' Bereich, in dem Content gezeichnet werden kann, d.h.:
    ' - Start = Borderwidth
    ' - Lnge recht bis zur gegnerischen ScrollBar bzw. gegenberliegenden Border
    Private m_PaintableRange As Range

    ' Bereich, in dem der anzuzeigende Inhalt sich relativ zum Nullpunkt
    ' (linke Begrenzung des Controls) befindet
    Private WithEvents m_ContentRange As New Range


    ' zugeordnete ScrollBar
    Private WithEvents m_ScrollBar As ScrollBar

    ' Referenz zum eigenen Control, das sichtbar oder unsichtbar gemacht wird, kann
    ' ScrollBar oder Panel sein
    Private m_OwnControl As Control

    ' gegnerisches Control(ScrollBar oder Panel), das seine Sichtbarkeit ndern kann
    ' und damit Einfluss auf die Zeichenbreite hat
    Private WithEvents m_OtherControl As Control

    ' Zeichenbreite des gegnerischen Controls
    Private m_OtherControlWidth As Integer

    ' enthlt sichtbare Breite der gegnerischen ScrollBar zum Ermitteln der eigenen 
    ' Ausdehnung. (wenn gegnerische ScrollBar unsichtbar, ist dieser Wert 0)
    Private m_OtherControlDelta As Integer = 0

    ' das kleine Rechteck, an dem sich die ScrollBars treffen
    Private WithEvents m_DividerControl As Control



    ' Rahmenbreite
    Private m_BorderWidth As Integer

    ' bekannte Breite der ScrollBar
    Private m_ControlWidth As Integer = 0

    ' Ausrichtung des Inhalts
    Private m_Alignment As StringAlignment = StringAlignment.Near

    ' gibt an, ob wenn das Control breiter als der Inhalt ist, der Inhalt zentriert wird
    Private m_CenterSmallContent As Boolean = False

    ' zuletzt mit manuellem Scrollen eingestellter Wert
    Private m_LastScrolledContentCenterValue As Single

    ' Gibt an, ob bereits einmal manuell gescrollt wurde.
    ' Wenn ja, wird beim Resize versucht, das Zentrum des nach dem letzten Scrollen
    ' sichtbaren Inhalts wieder zu erreichen
    Private m_HaveScrolled As Boolean = False

    ' Event nach manuellem Scrollen
    Public Event Scrolled()

#End Region

#Region " Properties "

    ''' <summary>
    ''' Lnge des anzuzeigenden Inhalts
    ''' </summary>
    WriteOnly Property ContentLength() As Integer
        Set(ByVal value As Integer)
            ' Eine nderung dieser Gre fhrt immer zu vollstndiger Neu-
            ' Ausrichtung
            Dim NewValue As Integer = Math.Max(0, value)
            m_ContentRange.Length = NewValue
            m_ScrollBar.Maximum = m_ContentRange.Length

            ' wenn die Breite der Zeichenflche nicht fr den anzuzeigenden Inhalt ausreicht, 
            ' ScrollBar anzeigen, im anderen Fall ScrollBar nicht anzeigen
            If m_PaintableRange.Length < m_ContentRange.Length Then
                m_OwnControl.Visible = True
            Else
                m_OwnControl.Visible = False
            End If
            ' Startwerte fr Inhalts- und ScrollBar-Position ermitteln
            InitContentRange()
        End Set
    End Property

    ''' <summary>
    ''' Ausrichtung des Inhalts
    ''' </summary>
    Property Alignment() As StringAlignment
        Get
            Return m_Alignment
        End Get
        Set(ByVal value As StringAlignment)

            m_Alignment = value
            ' Startwerte fr Inhalts- und ScrollBar-Position ermitteln
            InitContentRange()
        End Set
    End Property

    ''' <summary>
    '''  gibt an oder legt fest, ob wenn das Control breiter als der Inhalt ist, der Inhalt zentriert wird
    ''' </summary>
    Property CenterSmallContent() As Boolean
        Get
            Return m_CenterSmallContent
        End Get
        Set(ByVal value As Boolean)
            m_CenterSmallContent = value
            ' Startwerte fr Inhalts- und ScrollBar-Position ermitteln
            InitContentRange()
        End Set
    End Property

    ''' <summary>
    ''' Bereich (Startposition und Lnge), in dem der anzuzeigende Inhalt sich relativ zum Nullpunkt
    ''' (linke Begrenzung des Controls) befindet 
    ''' </summary>
    ReadOnly Property ContentRange() As Range
        Get
            Return m_ContentRange
        End Get
    End Property

    ''' <summary>
    ''' Bereich (Startposition und Lnge), in dem im Control Inhalt 
    ''' (oder ein Inhalts-Ausschnitt) zu sehen ist
    ''' </summary>
    ReadOnly Property ContentBackGroundRange() As Range
        Get
            Return m_PaintableRange.Intersection(m_ContentRange)
        End Get
    End Property

    ''' <summary>
    ''' Werte-Bereich: Startposition entspricht Pixel-Nr. des ersten sichtbaren Pixels, 
    ''' Lnge entspricht Pixel-Anzahl)
    ''' </summary>
    ReadOnly Property VisibleValuesRange() As Range
        Get
            Dim VisibleArea As Range = m_PaintableRange.Intersection(m_ContentRange)
            ' ContentRange.Start kann negativ sein
            Dim Indent As Integer = m_ContentRange.Start - m_BorderWidth ' (oder m_PaintableRange.Start)
            ' wenn Indent 0 oder grer ist, beginnt der Anzeigebereich bei 0,
            ' die Lnge des Anzeigebereichs entspricht der von VisibleArea
            ' wenn Indent < 0 ist,entspricht der erste angezeigte Wert dem 
            ' Kehrwert von Indent, die Lnge entspricht der von VisibleArea
            If Indent < 0 Then
                Dim FirstValue As Integer = Indent * -1
                Return New Range(FirstValue, VisibleArea.Length)
            Else
                Return New Range(0, VisibleArea.Length)
            End If
        End Get
    End Property

    ''' <summary>
    ''' Breite der ScrollBar in Pixeln
    ''' </summary>
    WriteOnly Property ControlWidth() As Integer
        Set(ByVal value As Integer)
            If m_ControlWidth <> value Then
                m_ControlWidth = value
                ' in der Folge zeichenbaren Bereich neu festlegen und Resize 
                ' fr die ScrollBar
                ChangePaintRange()
            End If
        End Set
    End Property


#End Region

#Region " Konstruktor "

    ' beim Initialisieren wird davon ausgegangen, dass beide ScrollBars sichtbar sind
    Public Sub New(ByVal ControlWidth As Integer, ByVal BorderWidth As Integer, ByVal ScrollBar As ScrollBar, _
    ByVal OwnControl As Control, ByVal OtherControl As Control, ByVal OtherControlWidth As Integer, ByVal DividerControl As Control)
        m_BorderWidth = BorderWidth
        m_PaintableRange = New Range(BorderWidth, ControlWidth - BorderWidth * 2)

        ' Die Sichtbarkeit der ScrollBars kann an zwei Stellen verndert werden:
        ' entweder durch nderung von ControlWidth oder durch nderung von ContentWidth
        m_ScrollBar = ScrollBar
        m_OwnControl = OwnControl
        m_OtherControl = OtherControl
        m_OtherControlWidth = OtherControlWidth
        m_DividerControl = DividerControl
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' berechnet Inhalt vollstndig neu und richtet Inhalt neu aus
    ''' </summary>
    Private Sub InitContentRange()

        ' Inhalt entweder links, rechts oder zentriert ausrichten
        SetStartScrollPosition()
        ' Info, dass nach dem Rcksetzen der Inhalts-Position noch nicht manuell gescrollt wurde
        m_HaveScrolled = False
    End Sub

    ''' <summary>
    ''' legt Startposition fr Content fest
    ''' </summary>
    Private Sub SetStartScrollPosition()
        ' und bei nderung einer der ContentRange-beeinflussenden Properties
        ' oder beim Resize ohne dass vorher manuell gescrollt wurde

        ' wenn der Bereich zum Zeichnen breiter als der Content ist und 
        ' schmaler Content zentriert werden soll
        If m_PaintableRange.Length > m_ContentRange.Length AndAlso m_CenterSmallContent Then
            ' Ausrichtung zentriert
            CenterRange()
        Else
            Select Case m_Alignment
                Case StringAlignment.Near
                    ' Ausrichtung links
                    m_ContentRange.Start = m_PaintableRange.Start
                Case StringAlignment.Center
                    ' Ausrichtung zentriert
                    CenterRange()
                Case StringAlignment.Far
                    ' Ausrichtung rechts
                    m_ContentRange.Start = m_PaintableRange.Ende - m_ContentRange.Length
            End Select
        End If
    End Sub

    ''' <summary>
    ''' zentriert Content
    ''' </summary>
    Private Sub CenterRange()
        ' Ausrichtung zentriert im PaintableRange
        Dim PaintableMidLength As Single = CSng(m_PaintableRange.Length) / 2.0F
        Dim ContentMidLength As Single = CSng(m_ContentRange.Length) / 2.0F
        Dim NewPosition As Single = PaintableMidLength - ContentMidLength
        NewPosition += CSng(m_PaintableRange.Start)

        ' -0.1F um zu erreichen, dass bei mittig liegendem Wert abgerundet wird
        m_ContentRange.Start = Math.Round(NewPosition - 0.1F)
    End Sub

    ''' <summary>
    ''' Verschieben des Content bei Resize des Controls
    ''' </summary>
    Private Sub PerformResize()
        ' Diese Methode wird verwendet bei normalem Resize des Controls 
        ' die ScrollBar-Position passt sich automatisch ber
        ' m_ContentRange.PositionChanged (Eventhandler hier in der Klasse) an


        ' wenn noch nicht manuell gescrollt wurde oder der Inhalt schmaler als 
        ' die Zeichenflche ist, verhlt sich das Zeichnen wie beim Start
        If Not m_HaveScrolled OrElse m_PaintableRange.Length > m_ContentRange.Length Then
            ' Position wie beim Initialisieren setzen
            SetStartScrollPosition()
        Else

            ' hier ist der Content breiter als die Zeichenflche, also soll
            ' sich ContentPosition am zuletzt per ScrollBar eingestellten Wert orientieren
            ' ContentPosition darf sich allerdings nur im Rahmen der erlaubten Grenzen bewegen
            Dim MinContentPosition As Integer = m_PaintableRange.Ende - m_ContentRange.Length
            Dim MaxContentPosition As Integer = m_PaintableRange.Start

            ' dies entsprche dem zuletzt in der Mitte des Controls sichtbaren Pixel
            ' Der Startwert von ContentRange soll nun so ausgerichtet werden, dass dieser Wert 
            ' wieder erreicht wird.
            Dim Zielwert As Single = m_LastScrolledContentCenterValue
            Dim ActiveWert As Single = VisibleValuesRange.Center
            Dim Delta As Integer = Zielwert - ActiveWert

            Dim ZielContentLeftValue As Integer = m_ContentRange.Start - Delta

            ZielContentLeftValue = Math.Max(ZielContentLeftValue, MinContentPosition)
            ZielContentLeftValue = Math.Min(ZielContentLeftValue, MaxContentPosition)
            ' Startposition fr ContentRange festlegen
            ' (in der Folge tritt m_ContentRange.PositionChanged ein)
            m_ContentRange.Start = ZielContentLeftValue
        End If
    End Sub


    ''' <summary>
    ''' Eventhandler fr manuelles Scrollen der ScrollBar
    ''' </summary>
    Private Sub m_ScrollBar_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles m_ScrollBar.Scroll
        ' mit der ScrollBar erreichter Wert fr ContentStart
        Dim NewContentStart As Integer = (e.NewValue - m_BorderWidth) * -1
        ' Nur wenn dieser Wert sich vom aktuellen unterscheidet
        ' (Scroll-Events werden anscheinend meehrfach auch bei gleichem Wert fr Value geliefert)
        If m_ContentRange.Start <> NewContentStart Then
            ' wird der neue Wert bernommen,
            m_ContentRange.Start = NewContentStart
            ' die aktuell zentral im sichtbaren Bereich liegende Position fr eventuell folgendes 
            ' Resize fest gehalten
            m_LastScrolledContentCenterValue = VisibleValuesRange.Center
            ' Flag, das manuell gescrollt wurde, wird auf True gesetzt
            m_HaveScrolled = True
            ' Event fr den Align-Container, damit dieser die Positionen der enthaltenen Controls anpassen kann
            RaiseEvent Scrolled()
        End If

    End Sub

    ''' <summary>
    ''' Eventhandler fr das PositionChanged des ContentRange-Objektes
    ''' </summary>
    Private Sub m_ContentRange_PositionChanged() Handles m_ContentRange.PositionChanged
        ' positioniert die ScrollBar neu
        Dim NewScrollBarValue As Integer = Math.Max(m_ContentRange.Start * -1 + m_BorderWidth, 0)
        m_ScrollBar.Value = NewScrollBarValue
    End Sub


    ''' <summary>
    ''' Eventhandler fr das VisibleChanged-Event der gegnerischen ScrollBar
    ''' </summary>
    Private Sub m_OtherControl_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_OtherControl.VisibleChanged
        ' Delta-Wert neu festlegen
        If m_OtherControl.Visible Then
            m_OtherControlDelta = m_OtherControlWidth
        Else
            m_OtherControlDelta = 0
        End If
        ' Eigenen Zeichenbereich neu ermitteln und ggf. die Scroll-Position 
        ' anpassen
        ChangePaintRange()
        ' wenn beide ScrollBars sichtbar sind, Divider sichtbar machen,
        ' im anderen Fall Divider unsichtbar machen
        If m_OwnControl.Visible AndAlso m_OtherControl.Visible Then
            If m_OwnControl Is m_ScrollBar Then
                m_ScrollBar.BringToFront()
            End If
            m_DividerControl.Visible = True
        Else
            m_DividerControl.Visible = False
        End If
    End Sub

    ''' <summary>
    ''' legt den Zeichenbereich neu fest
    ''' </summary>
    Private Sub ChangePaintRange()
        ' Zeichenbereichs-Breite = Breite des Controls - (ggf.) Breite der gegnerischen ScrollBar - 2x Rahmenbreite
        Dim NewLength As Integer = Math.Max(m_ControlWidth - m_OtherControlDelta - m_BorderWidth * 2, 0)

        If m_PaintableRange.Length <> NewLength Then
            m_PaintableRange.Length = NewLength
            ' wenn der Zeichenbereich schmaler als der anzuzeigende Inhalt ist,
            ' ScrollBar anzeigen, ansonsten ScrollBar nicht anzeigen
            If m_PaintableRange.Length < m_ContentRange.Length Then
                m_OwnControl.Visible = True
            Else
                m_OwnControl.Visible = False
            End If
            ' LargeChange der ScrollBar anpassen
            m_ScrollBar.LargeChange = m_PaintableRange.Length + 1
            ' Anzeige-Positionen fr Resize ermitteln
            PerformResize()
        End If
    End Sub

#End Region

End Class
