''' <summary>
''' enfaches Control, das 50 x 50 farbige Felder darstellen kann
''' </summary>
''' <remarks></remarks>
Public Class PixelGrid
    Inherits Control

#Region " Private Felder "

    ' zugrunde liegende Bilddatei als Datenbasis
    Private m_Datei As BildDatei
    ' Liste mit verschiedenfarbigen Brushes zum Zeichnen
    Private m_Brushes As New List(Of Brush)
    ' aktiver Farb-Index
    Private m_ActiveColorIndex As Integer = 1
    ' Kennzeichnet das Farbrechteck (Spalten- und Zeilen-Index), an dem sich die Maus
    ' im MouseDown-Zustand zuletzt befunden hat
    Private m_LastMovePoint As New Point(-1, -1)

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt den aktiven Farbindex zurck oder legt diesen fest
    ''' </summary>
    Property ActiveColorIndex() As Integer
        Get
            Return m_ActiveColorIndex
        End Get
        Set(ByVal value As Integer)
            If value < 0 OrElse value > 7 Then
                Throw New Exception("Nicht zulssiger Wert fr Farbindex")
            End If
            m_ActiveColorIndex = value
        End Set
    End Property

    ''' <summary>
    ''' gibt die zugrunde liegende Bilddatei zurck oder legt diese fest
    ''' </summary>
    Property Datei() As BildDatei
        Get
            Return m_Datei
        End Get
        Set(ByVal value As BildDatei)
            If m_Datei IsNot value Then
                m_Datei = value
                ' neu zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New()
        ' Liste mit Brushes in Reihenfolge der Bedeutung der Farbindizes fllen
        m_Brushes.Add(Brushes.White)
        m_Brushes.Add(Brushes.Red)
        m_Brushes.Add(Brushes.Orange)
        m_Brushes.Add(Brushes.Yellow)
        m_Brushes.Add(Brushes.LimeGreen)
        m_Brushes.Add(Brushes.Turquoise)
        m_Brushes.Add(Brushes.Blue)
        m_Brushes.Add(Brushes.Black)
        ' Gre festlegen
        Me.Size = New Size(499, 499)
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um den Inhalt zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        Dim g As Graphics = e.Graphics
        ' Hintergrund mit Control-Farbe fllen
        g.Clear(SystemColors.Control)

        ' ermitteln, welcher Wertebereich fr die waagerechten und senkrechten Striche 
        ' im Zeichenbereich liegt
        Dim NumxStart As Integer = Math.Max(e.ClipRectangle.Left \ 10, 0)
        Dim NumxEnde As Integer = Math.Min((e.ClipRectangle.Right \ 10), 49)
        Dim NumyStart As Integer = Math.Max(e.ClipRectangle.Top \ 10, 0)
        Dim NumyEnde As Integer = Math.Min((e.ClipRectangle.Bottom \ 10), 49)


        Dim xStart As Integer = NumxStart * 10 - 1
        Dim xEnde As Integer = NumxEnde * 10
        Dim yStart As Integer = NumyStart * 10 - 1
        Dim yEnde As Integer = NumyEnde * 10

        ' senkrechte Linien zeichnen
        For I As Integer = xStart To xEnde Step 10
            g.DrawLine(Pens.LightGray, I, 0, I, Me.Height)
        Next
        ' waagerechte Linien zeichnen
        For J As Integer = yStart To yEnde Step 10
            g.DrawLine(Pens.LightGray, 0, J, Me.Width, J)
        Next

        ' wenn eine Datei angegeben wurd
        If m_Datei IsNot Nothing Then
            ' alle im Zeichenbereich liegenden Rechtecke mit passenden Farben ausfllen
            For I As Integer = NumxStart To NumxEnde
                For J As Integer = NumyStart To NumyEnde
                    Dim Rect As New Rectangle(I * 10, J * 10, 9, 9)
                    ' Das Ermitteln der x-und y- Grenzen oben war nicht exakt
                    ' also durchaus nochmal kontrollieren, ob dieses Rechteck wirklich zu zeichnen ist
                    If e.ClipRectangle.IntersectsWith(Rect) Then
                        g.FillRectangle(m_Brushes(m_Datei.ColorIndizes(I, J)), Rect)
                    End If
                Next
            Next
        End If

    End Sub

    ''' <summary>
    ''' Methode bei OnMouseMove im Control
    ''' </summary>
    Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseMove(e)
        ' wenn notwendig, zeichnen
        Draw(False, e)
    End Sub

    ''' <summary>
    ''' ndert abhngig von MousePosition und MouseButton einen einzelnen Pixel
    ''' </summary>
    Private Sub Draw(ByVal MouseDown As Boolean, ByVal e As System.Windows.Forms.MouseEventArgs)

        ' wenn der links BouseButton gedrckt ist
        If e.Button = Windows.Forms.MouseButtons.Left Then
            ' Koordinaten innerhalb der Farb-Rechtecke ermitteln
            Dim NewX, NewY As Integer
            NewX = Math.Max(Math.Min(e.X \ 10, 49), 0)
            NewY = Math.Max(Math.Min(e.Y \ 10, 49), 0)

            ' Wenn diese mit den zuletzt bekannten MouseMove-Koordinaten nicht bereinstimmen
            If MouseDown OrElse m_LastMovePoint.X <> NewX OrElse m_LastMovePoint.Y <> NewY Then
                ' neue Koordinaten merken
                m_LastMovePoint = New Point(NewX, NewY)
                ' aktiven Farbindex in der Datei an den entsprechenden Koordinaten eintragen
                m_Datei.ColorIndizes(NewX, NewY) = m_ActiveColorIndex
                ' Control nur zum Neuzeichnen dieses kleinen Farb-Rechtecks veranlassen
                Me.Invalidate(New Rectangle(NewX * 10, NewY * 10, 9, 9))
            End If
        End If
    End Sub

    ''' <summary>
    ''' Methode bei OnMouseUp im Control
    ''' </summary>
    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseUp(e)
        ' LastMovePoint zurck setzen
        m_LastMovePoint = New Point(-1, -1)
    End Sub

    ''' <summary>
    ''' Methode bei OnMouseDown im Control
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        Draw(True, e)

    End Sub

    ''' <summary>
    ''' Entfernt den Inhalt einer Farb-Datei
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Clear()
        ' setzt eigentlich nur alle Farb-Indizes auf den fr weiss
        If m_Datei IsNot Nothing Then
            For I As Integer = 0 To 49
                For J As Integer = 0 To 49
                    m_Datei.ColorIndizes(I, J) = 0
                Next
            Next
            Invalidate()
        End If
    End Sub

#End Region

End Class
